import 'package:flutter/material.dart';

import 'package:puzzlegame/entities/moving_tile.dart';
import 'package:puzzlegame/provider/data.dart';
import 'package:puzzlegame/utils/game_utils.dart';

class Game {
  static void toggleDisplayTipImage(Data myProvider) {
    myProvider.updateIsTipImageDisplayed(!myProvider.displayTipImage);
  }

  static Widget buildTilesetWidget(Data myProvider) {
    final List<MovingTile> tiles = myProvider.tiles;

    final Color borderColor =
        GameUtils.checkTilesetIsCleared(tiles) ? Colors.green : Colors.orange;
    int tileIndex = 0;

    final Table tileset = Table(
      defaultColumnWidth: const IntrinsicColumnWidth(),
      border: TableBorder.all(
        color: Colors.black,
        style: BorderStyle.solid,
        width: 2,
      ),
      children: [
        for (int row = 0; row < myProvider.tilesCount; row++)
          TableRow(
            children: [
              for (int col = 0; col < myProvider.tilesCount; col++)
                Column(children: [tiles[tileIndex++].widget(myProvider)]),
            ],
          ),
      ],
    );

    return Container(
      margin: const EdgeInsets.all(8),
      padding: const EdgeInsets.all(8),
      decoration: BoxDecoration(
        color: borderColor,
        borderRadius: BorderRadius.circular(8),
        border: Border.all(
          color: borderColor,
          width: 8,
        ),
      ),
      child: tileset,
    );
  }

  static Widget buildTipWidget(Data myProvider) {
    return Container(
      margin: const EdgeInsets.all(2),
      padding: const EdgeInsets.all(2),
      child: Table(
        defaultColumnWidth: const IntrinsicColumnWidth(),
        children: [
          TableRow(
            children: [
              const Column(
                children: [
                  Image(
                    image: AssetImage('assets/images/placeholder.png'),
                    fit: BoxFit.fill,
                  ),
                ],
              ),
              Column(
                children: [
                  TextButton(
                    child: Container(
                      decoration: BoxDecoration(
                        color: Colors.blue,
                        borderRadius: BorderRadius.circular(4),
                        border: Border.all(
                          color: Colors.blue,
                          width: 4,
                        ),
                      ),
                      child: Image(
                        image: AssetImage(
                          myProvider.displayTipImage
                              ? GameUtils.getImageAssetName(myProvider.selectedImage)
                              : 'assets/icons/tip_hidden.png',
                        ),
                        fit: BoxFit.contain,
                      ),
                    ),
                    onPressed: () => Game.toggleDisplayTipImage(myProvider),
                  ),
                ],
              ),
              const Column(
                children: [
                  Image(
                    image: AssetImage('assets/images/placeholder.png'),
                    fit: BoxFit.fill,
                  ),
                ],
              ),
            ],
          ),
        ],
      ),
    );
  }

  static Widget buildGameWidget(Data myProvider) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        const SizedBox(height: 8),
        buildTopIndicatorWidget(myProvider),
        const SizedBox(height: 2),
        Expanded(
          child: buildGameBoard(myProvider),
        ),
        const SizedBox(height: 2),
        Container(
          child: GameUtils.checkTilesetIsCleared(myProvider.tiles)
              ? Game.buildWinMessage(myProvider)
              : Game.buildTipWidget(myProvider),
        ),
      ],
    );
  }

  static Widget buildGameBoard(Data myProvider) {
    return Container(
      margin: const EdgeInsets.all(4),
      padding: const EdgeInsets.all(4),
      child: Column(
        children: [
          buildTilesetWidget(myProvider),
        ],
      ),
    );
  }

  static Widget buildTopIndicatorWidget(Data myProvider) {
    return Table(
      children: [
        TableRow(
          children: [
            Column(children: [
              Text(
                myProvider.movesCount.toString(),
                style: const TextStyle(
                  fontSize: 40,
                  fontWeight: FontWeight.w600,
                  color: Colors.black,
                ),
              ),
            ]),
            const Column(children: []),
          ],
        ),
      ],
    );
  }

  static Widget buildWinMessage(Data myProvider) {
    return Container(
      margin: const EdgeInsets.all(2),
      padding: const EdgeInsets.all(2),
      child: Table(
        defaultColumnWidth: const IntrinsicColumnWidth(),
        children: [
          TableRow(
            children: [
              const Column(
                children: [
                  Image(
                    image: AssetImage('assets/icons/game_win.png'),
                    fit: BoxFit.fill,
                  ),
                ],
              ),
              Column(
                children: [
                  TextButton(
                    child: const Image(
                      image: AssetImage('assets/icons/button_back.png'),
                      fit: BoxFit.fill,
                    ),
                    onPressed: () => GameUtils.resetGame(myProvider),
                  ),
                ],
              ),
              const Column(
                children: [
                  Image(
                    image: AssetImage('assets/icons/game_win.png'),
                    fit: BoxFit.fill,
                  ),
                ],
              ),
            ],
          ),
        ],
      ),
    );
  }

  static Widget buildShufflingIndicatorWidget() {
    return const Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        Text(
          '⏳',
          style: TextStyle(
            fontSize: 60,
            fontWeight: FontWeight.w600,
            color: Colors.black,
          ),
        ),
        SizedBox(height: 20),
        CircularProgressIndicator(),
      ],
    );
  }
}
